"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoveNatGatewaysTrick = void 0;
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const chalk_1 = __importDefault(require("chalk"));
class RemoveNatGatewaysTrick {
    constructor() {
        this.ec2Client = new aws_sdk_1.default.EC2();
    }
    getMachineName() {
        return RemoveNatGatewaysTrick.machineName;
    }
    getDisplayName() {
        return 'Remove NAT Gateways';
    }
    canBeConcurrent() {
        return true;
    }
    async conserve(subListr, dryRun) {
        const natGateways = await this.listNatGateways();
        const currentState = await this.getCurrentState(natGateways);
        for (const natGateway of currentState) {
            subListr.add({
                title: `${chalk_1.default.blueBright(natGateway.id)} / ${natGateway.tags
                    .filter(t => { var _a; return ((_a = t.Key) === null || _a === void 0 ? void 0 : _a.toString().toLowerCase()) === 'name'; })
                    .join(' ')}`,
                task: (ctx, task) => this.conserveNatGateway(task, dryRun, natGateway),
            });
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const natGateway of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(natGateway.id),
                task: (ctx, task) => this.restoreNatGateway(task, dryRun, natGateway),
            });
        }
    }
    async conserveNatGateway(task, dryRun, natGateway) {
        if (dryRun) {
            task.skip(`Skipped due to dry-run`);
        }
        else {
            task.output = 'Deleting NAT gateway...';
            await this.ec2Client
                .deleteNatGateway({
                NatGatewayId: natGateway.id,
            })
                .promise();
            task.output = 'NAT Gateway Deleted';
        }
    }
    async restoreNatGateway(task, dryRun, natGateway) {
        var _a;
        if (dryRun) {
            task.skip(`Skipped due to dry-run`);
        }
        else {
            task.output = 'Creating NAT gateway...';
            const result = await this.ec2Client
                .createNatGateway({
                AllocationId: natGateway.allocationIds && natGateway.allocationIds[0],
                SubnetId: natGateway.subnetId,
                TagSpecifications: [
                    {
                        ResourceType: 'natgateway',
                        Tags: natGateway.tags,
                    },
                ],
            })
                .promise();
            task.output = 'Waiting for NAT Gateway to be available...';
            await this.ec2Client.waitFor('natGatewayAvailable', {
                NatGatewayIds: [((_a = result.NatGateway) === null || _a === void 0 ? void 0 : _a.NatGatewayId) || ''],
            });
            task.output = 'NAT Gateway Created';
        }
    }
    async getCurrentState(natGatewayList) {
        return Promise.all(natGatewayList.map(async (natGateway) => {
            var _a;
            if (!natGateway.NatGatewayId ||
                !natGateway.VpcId ||
                !natGateway.SubnetId ||
                !natGateway.Tags) {
                throw new Error(`Unexpected values on Nat Gateway: ${JSON.stringify(natGateway)}`);
            }
            return {
                id: natGateway.NatGatewayId,
                vpcId: natGateway.VpcId,
                subnetId: natGateway.SubnetId,
                allocationIds: ((_a = natGateway.NatGatewayAddresses) === null || _a === void 0 ? void 0 : _a.map(a => a.AllocationId || '')) ||
                    [],
                tags: natGateway.Tags,
            };
        }));
    }
    async listNatGateways() {
        return ((await this.ec2Client.describeNatGateways().promise()).NatGateways || []);
    }
}
exports.RemoveNatGatewaysTrick = RemoveNatGatewaysTrick;
RemoveNatGatewaysTrick.machineName = 'remove-nat-gateways';
//# sourceMappingURL=remove-nat-gateways.trick.js.map