"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapshotRemoveElasticacheRedisTrick = void 0;
const lodash_1 = __importDefault(require("lodash"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const chalk_1 = __importDefault(require("chalk"));
class SnapshotRemoveElasticacheRedisTrick {
    constructor() {
        this.elcClient = new aws_sdk_1.default.ElastiCache();
    }
    getMachineName() {
        return SnapshotRemoveElasticacheRedisTrick.machineName;
    }
    getDisplayName() {
        return 'Snapshot and remove ElastiCache Redis';
    }
    canBeConcurrent() {
        return true;
    }
    async conserve(subListr, dryRun) {
        const replicationGroups = await this.listReplicationGroups();
        const currentState = await this.getCurrentState(replicationGroups);
        if (currentState.length > 0) {
            for (const replicationGroup of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(replicationGroup.id),
                    task: (ctx, task) => this.conserveReplicationGroup(task, dryRun, replicationGroup),
                });
            }
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const singleNodeCluster of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(singleNodeCluster.id),
                task: (ctx, task) => this.restoreReplicationGroup(task, dryRun, singleNodeCluster),
            });
        }
    }
    async conserveReplicationGroup(task, dryRun, replicationGroupState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (replicationGroupState.status === 'available') {
            task.output = 'Taking a snapshot and deleting replication group...';
            await this.elcClient
                .deleteReplicationGroup({
                ReplicationGroupId: replicationGroupState.id,
                FinalSnapshotIdentifier: replicationGroupState.snapshotName,
            })
                .promise();
            task.output =
                'Waiting for snapshot to be created, and replication group to be deleted...';
            await this.elcClient
                .waitFor('replicationGroupDeleted', {
                ReplicationGroupId: replicationGroupState.id,
                $waiter: {
                    delay: 30,
                    maxAttempts: 100,
                },
            })
                .promise();
            task.output = `Snapshot ${replicationGroupState.snapshotName} taken and replication group deleted successfully`;
        }
        else {
            task.skip(`Skipped, current state is not "available" it is "${replicationGroupState.status}" instead`);
        }
    }
    async restoreReplicationGroup(task, dryRun, replicationGroupState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (replicationGroupState.status === 'available') {
            if (await this.isReplicationGroupAvailable(replicationGroupState)) {
                task.skip('Already exists');
                return;
            }
            task.output = `Creating replication group (snapshot: ${replicationGroupState.snapshotName})...`;
            await this.elcClient
                .createReplicationGroup(replicationGroupState.createParams)
                .promise();
            task.output = `Waiting for replication group to be available (snapshot: ${replicationGroupState.snapshotName})...`;
            await this.elcClient
                .waitFor('replicationGroupAvailable', {
                ReplicationGroupId: replicationGroupState.id,
                $waiter: {
                    delay: 30,
                    maxAttempts: 100,
                },
            })
                .promise();
            task.output = `Deleting snapshot ${replicationGroupState.snapshotName}...`;
            await this.elcClient
                .deleteSnapshot({
                SnapshotName: replicationGroupState.snapshotName,
            })
                .promise();
            task.output = 'Restored successfully';
        }
        else {
            task.skip(`Skipped, previous state was not "available" it was "${replicationGroupState.status}" instead`);
        }
    }
    async getCurrentState(replicationGroups) {
        return Promise.all(replicationGroups.map(async (replicationGroup) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w;
            if (replicationGroup.ARN === undefined) {
                throw new Error(`Unexpected error: ARN is missing for ElastiCache replication group`);
            }
            if (replicationGroup.ReplicationGroupId === undefined) {
                throw new Error(`Unexpected error: ReplicationGroupId is missing for ElastiCache replication group`);
            }
            if (replicationGroup.Status === undefined) {
                throw new Error(`Unexpected error: Status is missing for ElastiCache replication group`);
            }
            if (replicationGroup.AuthTokenEnabled) {
                throw new Error(`Cannot conserve an AuthToken protected Redis Cluster because on restore token will change`);
            }
            const cacheClusters = await this.getCacheClusters(replicationGroup);
            const now = new Date();
            const snapshotName = `${replicationGroup.ReplicationGroupId}-${now.getFullYear()}${now.getMonth()}${now.getDay()}${now.getHours()}${now.getMinutes()}`;
            const nodeGroups = ((_a = replicationGroup.NodeGroups) === null || _a === void 0 ? void 0 : _a.map(n => {
                var _a;
                return ({
                    NodeGroupId: n.NodeGroupId,
                    ReplicaCount: (_a = n.NodeGroupMembers) === null || _a === void 0 ? void 0 : _a.length,
                    Slots: n.Slots,
                });
            })) || [];
            const createParams = {
                // Required
                ReplicationGroupId: replicationGroup.ReplicationGroupId,
                ReplicationGroupDescription: replicationGroup.Description ||
                    `Restored by aws-cost-saver from snapshot: ${snapshotName}`,
                SnapshotName: snapshotName,
                // Replication Group Configs
                AtRestEncryptionEnabled: replicationGroup.AtRestEncryptionEnabled,
                TransitEncryptionEnabled: replicationGroup.TransitEncryptionEnabled,
                NumCacheClusters: nodeGroups.length > 1 ? undefined : cacheClusters.length,
                NumNodeGroups: nodeGroups.length > 1 ? nodeGroups.length : undefined,
                MultiAZEnabled: replicationGroup.MultiAZ !== undefined &&
                    replicationGroup.MultiAZ === 'enabled',
                NodeGroupConfiguration: nodeGroups.length > 1 ? nodeGroups : undefined,
                GlobalReplicationGroupId: (_b = replicationGroup.GlobalReplicationGroupInfo) === null || _b === void 0 ? void 0 : _b.GlobalReplicationGroupId,
                KmsKeyId: replicationGroup.KmsKeyId,
                AutomaticFailoverEnabled: replicationGroup.AutomaticFailover !== undefined &&
                    ['enabled', 'enabling'].includes(replicationGroup.AutomaticFailover),
                Port: (((_c = cacheClusters[0]) === null || _c === void 0 ? void 0 : _c.CacheNodes) && ((_f = (_e = (_d = cacheClusters[0]) === null || _d === void 0 ? void 0 : _d.CacheNodes[0]) === null || _e === void 0 ? void 0 : _e.Endpoint) === null || _f === void 0 ? void 0 : _f.Port)) ||
                    (replicationGroup.NodeGroups && ((_h = (_g = replicationGroup.NodeGroups[0]) === null || _g === void 0 ? void 0 : _g.PrimaryEndpoint) === null || _h === void 0 ? void 0 : _h.Port)),
                Tags: await this.getTags(((_j = cacheClusters[0]) === null || _j === void 0 ? void 0 : _j.ARN) || ''),
                // Cache Clusters Configs
                PreferredCacheClusterAZs: nodeGroups.length > 1
                    ? undefined
                    : lodash_1.default.uniq(cacheClusters
                        .map(c => c.PreferredAvailabilityZone || '')
                        .filter(az => az !== '')),
                SecurityGroupIds: lodash_1.default.uniq([].concat(...cacheClusters.map(c => c.SecurityGroups
                    ? c.SecurityGroups.map(s => s.SecurityGroupId || '')
                    : []))).filter(sg => sg !== ''),
                CacheSecurityGroupNames: lodash_1.default.uniq([].concat(...cacheClusters.map(c => c.CacheSecurityGroups
                    ? c.CacheSecurityGroups.map(s => s.CacheSecurityGroupName || '')
                    : []))).filter(sg => sg !== ''),
                // Shared Cache Cluster Configs
                AutoMinorVersionUpgrade: (_k = cacheClusters[0]) === null || _k === void 0 ? void 0 : _k.AutoMinorVersionUpgrade,
                CacheNodeType: (_l = cacheClusters[0]) === null || _l === void 0 ? void 0 : _l.CacheNodeType,
                SnapshotRetentionLimit: (_m = cacheClusters[0]) === null || _m === void 0 ? void 0 : _m.SnapshotRetentionLimit,
                Engine: (_o = cacheClusters[0]) === null || _o === void 0 ? void 0 : _o.Engine,
                EngineVersion: (_p = cacheClusters[0]) === null || _p === void 0 ? void 0 : _p.EngineVersion,
                CacheParameterGroupName: (_r = (_q = cacheClusters[0]) === null || _q === void 0 ? void 0 : _q.CacheParameterGroup) === null || _r === void 0 ? void 0 : _r.CacheParameterGroupName,
                CacheSubnetGroupName: (_s = cacheClusters[0]) === null || _s === void 0 ? void 0 : _s.CacheSubnetGroupName,
                NotificationTopicArn: (_u = (_t = cacheClusters[0]) === null || _t === void 0 ? void 0 : _t.NotificationConfiguration) === null || _u === void 0 ? void 0 : _u.TopicArn,
                SnapshotWindow: (_v = cacheClusters[0]) === null || _v === void 0 ? void 0 : _v.SnapshotWindow,
                PreferredMaintenanceWindow: (_w = cacheClusters[0]) === null || _w === void 0 ? void 0 : _w.PreferredMaintenanceWindow,
            };
            return {
                id: replicationGroup.ReplicationGroupId,
                status: replicationGroup.Status,
                snapshotName,
                createParams,
            };
        }));
    }
    async isReplicationGroupAvailable(replicationGroupState) {
        try {
            const result = await this.elcClient
                .describeReplicationGroups({
                ReplicationGroupId: replicationGroupState.id,
            })
                .promise();
            if (result.ReplicationGroups && result.ReplicationGroups.length > 0) {
                return true;
            }
        }
        catch (error) {
            // Skip
        }
        return false;
    }
    async listReplicationGroups() {
        return ((await this.elcClient.describeReplicationGroups({}).promise())
            .ReplicationGroups || []);
    }
    async getCacheClusters(replicationGroup) {
        return ((await this.elcClient
            .describeCacheClusters({
            CacheClusterId: replicationGroup.SnapshottingClusterId,
            ShowCacheNodeInfo: true,
        })
            .promise()).CacheClusters || []);
    }
    async getTags(arn) {
        return ((await this.elcClient
            .listTagsForResource({
            ResourceName: arn,
        })
            .promise()).TagList || []);
    }
}
exports.SnapshotRemoveElasticacheRedisTrick = SnapshotRemoveElasticacheRedisTrick;
SnapshotRemoveElasticacheRedisTrick.machineName = 'snapshot-remove-elasticache-redis';
//# sourceMappingURL=snapshot-remove-elasticache-redis.trick.js.map