"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StopFargateEcsServicesTrick = void 0;
const lodash_1 = __importDefault(require("lodash"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const listr_1 = __importDefault(require("listr"));
const chalk_1 = __importDefault(require("chalk"));
class StopFargateEcsServicesTrick {
    constructor() {
        this.ecsClient = new aws_sdk_1.default.ECS();
        this.aasClient = new aws_sdk_1.default.ApplicationAutoScaling();
    }
    getMachineName() {
        return StopFargateEcsServicesTrick.machineName;
    }
    getDisplayName() {
        return 'Stop Fargate ECS Services';
    }
    canBeConcurrent() {
        return true;
    }
    async conserve(subListr, dryRun) {
        const clustersArn = await this.listClusters();
        const currentState = await this.getCurrentState(clustersArn);
        for (const cluster of currentState) {
            for (const service of cluster.services) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(this.getEcsServiceResourceId(cluster.arn, service.arn))}`,
                    task: () => new listr_1.default([
                        {
                            title: 'Desired count',
                            task: (ctx, task) => this.conserveService(task, dryRun, cluster, service),
                        },
                        {
                            title: 'Auto scaling',
                            task: (ctx, task) => this.conserveScalableTargets(task, dryRun, cluster, service),
                        },
                    ], {
                        exitOnError: false,
                        concurrent: true,
                        // @ts-ignore
                        collapse: false,
                    }),
                });
            }
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const cluster of originalState) {
            for (const service of cluster.services) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(this.getEcsServiceResourceId(cluster.arn, service.arn))}`,
                    task: () => new listr_1.default([
                        {
                            title: 'Desired count',
                            task: (ctx, task) => this.restoreService(task, dryRun, cluster, service),
                        },
                        {
                            title: 'Auto scaling',
                            task: (ctx, task) => this.restoreScalableTargets(task, dryRun, cluster, service),
                        },
                    ], {
                        exitOnError: false,
                        concurrent: true,
                        // @ts-ignore
                        collapse: true,
                    }),
                });
            }
        }
    }
    async conserveService(task, dryRun, clusterState, serviceState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (serviceState.desired > 0) {
            task.output = `Updating desired count to zero...`;
            await this.ecsClient
                .updateService({
                cluster: clusterState.arn,
                service: serviceState.arn,
                desiredCount: 0,
            })
                .promise();
            task.output = `Waiting for service to scale down to zero...`;
            await this.ecsClient.waitFor('servicesStable', {
                cluster: clusterState.arn,
                services: [serviceState.arn],
            });
            task.output = 'Set desired count to zero';
        }
        else {
            task.skip(`Skipped, desired count is already zero`);
        }
    }
    async conserveScalableTargets(task, dryRun, clusterState, serviceState) {
        if (!serviceState.scalableTargets ||
            serviceState.scalableTargets.length === 0) {
            task.skip('No scalable targets defined');
            return;
        }
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else {
            for (const scalableTarget of serviceState.scalableTargets) {
                const resourceId = this.getEcsServiceResourceId(clusterState.arn, serviceState.arn);
                await this.aasClient
                    .registerScalableTarget({
                    ServiceNamespace: scalableTarget.namespace,
                    ResourceId: resourceId,
                    ScalableDimension: scalableTarget.scalableDimension,
                    MinCapacity: 0,
                    MaxCapacity: 0,
                })
                    .promise();
            }
        }
    }
    async restoreService(task, dryRun, clusterState, serviceState) {
        if (dryRun) {
            task.skip(`Skipped due to dry-run`);
        }
        else if (serviceState.desired > 0) {
            task.output = `Updating desired count to ${serviceState.desired}...`;
            await this.ecsClient
                .updateService({
                cluster: clusterState.arn,
                service: serviceState.arn,
                desiredCount: serviceState.desired,
            })
                .promise();
            task.output = `Waiting for service to reach ${serviceState.desired} desired tasks...`;
            await this.ecsClient.waitFor('servicesStable', {
                cluster: clusterState.arn,
                services: [serviceState.arn],
            });
            task.output = `Restored desired count to ${serviceState.desired}`;
        }
        else {
            task.skip(`Skipped, desired count was previously zero`);
        }
    }
    async restoreScalableTargets(task, dryRun, clusterState, serviceState) {
        if (!serviceState.scalableTargets ||
            serviceState.scalableTargets.length === 0) {
            task.skip('No scalable targets defined');
            return;
        }
        for (const scalableTarget of serviceState.scalableTargets) {
            const resourceId = this.getEcsServiceResourceId(clusterState.arn, serviceState.arn);
            if (dryRun) {
                task.skip('Skipped due to dry-run');
            }
            else {
                await this.aasClient
                    .registerScalableTarget({
                    ServiceNamespace: scalableTarget.namespace,
                    ResourceId: resourceId,
                    ScalableDimension: scalableTarget.scalableDimension,
                    MinCapacity: scalableTarget.min,
                    MaxCapacity: scalableTarget.max,
                })
                    .promise();
            }
        }
    }
    async getCurrentState(clustersArn) {
        return Promise.all(clustersArn.map(async (clusterArn) => {
            const services = await this.describeAllServices(clusterArn);
            return {
                arn: clusterArn,
                services: await Promise.all(services.map(async (service) => {
                    if (service.serviceArn === undefined) {
                        throw new Error(`Unexpected error: serviceArn is missing for ECS service`);
                    }
                    if (service.desiredCount === undefined) {
                        throw new Error(`Unexpected error: desiredCount is missing for ECS service`);
                    }
                    const scalableTargets = await this.describeAllScalableTargets(service);
                    return {
                        arn: service.serviceArn,
                        desired: service.desiredCount,
                        scalableTargets: scalableTargets.map(st => ({
                            namespace: 'ecs',
                            resourceId: st.ResourceId,
                            scalableDimension: st.ScalableDimension,
                            min: st.MinCapacity,
                            max: st.MaxCapacity,
                        })),
                    };
                })),
            };
        }));
    }
    async listServices(clusterArn) {
        return ((await this.ecsClient
            .listServices({
            cluster: clusterArn,
            launchType: 'FARGATE',
        })
            .promise()).serviceArns || []);
    }
    async listClusters() {
        return (await this.ecsClient.listClusters({}).promise()).clusterArns || [];
    }
    async describeAllServices(clusterArn) {
        const servicesArn = await this.listServices(clusterArn);
        const chunks = lodash_1.default.chunk(servicesArn, 10);
        const result = [];
        for (const services of chunks) {
            const response = await this.ecsClient
                .describeServices({ services, cluster: clusterArn })
                .promise();
            result.push(...(response.services || []));
        }
        return result;
    }
    async describeAllScalableTargets(service) {
        if (!service.clusterArn) {
            throw new Error(`Unexpected missing clusterArn for ECS service: ${service}`);
        }
        if (!service.serviceArn) {
            throw new Error(`Unexpected missing serviceArn for ECS service: ${service}`);
        }
        const result = await this.aasClient
            .describeScalableTargets({
            ServiceNamespace: 'ecs',
            ResourceIds: [
                this.getEcsServiceResourceId(service.clusterArn, service.serviceArn),
            ],
            ScalableDimension: 'ecs:service:DesiredCount',
        })
            .promise();
        return result.ScalableTargets || [];
    }
    getEcsServiceResourceId(clusterArn, serviceArn) {
        const clusterName = clusterArn
            .toString()
            .split('/')
            .pop();
        const serviceName = serviceArn
            .toString()
            .split('/')
            .pop();
        return `service/${clusterName}/${serviceName}`;
    }
}
exports.StopFargateEcsServicesTrick = StopFargateEcsServicesTrick;
StopFargateEcsServicesTrick.machineName = 'stop-fargate-ecs-services';
//# sourceMappingURL=stop-fargate-ecs-services.trick.js.map