"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StopRdsDatabaseInstancesTrick = void 0;
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const chalk_1 = __importDefault(require("chalk"));
class StopRdsDatabaseInstancesTrick {
    constructor() {
        this.rdsClient = new aws_sdk_1.default.RDS();
    }
    getMachineName() {
        return StopRdsDatabaseInstancesTrick.machineName;
    }
    getDisplayName() {
        return 'Stop RDS Database Instances';
    }
    canBeConcurrent() {
        return true;
    }
    async conserve(subListr, dryRun) {
        const databases = await this.listDatabases();
        const currentState = await this.getCurrentState(databases);
        if (currentState.length > 0) {
            for (const database of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(database.identifier),
                    task: (ctx, task) => this.conserveDatabase(task, dryRun, database),
                });
            }
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const database of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(database.identifier),
                task: (ctx, task) => this.restoreDatabase(task, dryRun, database),
            });
        }
    }
    async conserveDatabase(task, dryRun, databaseState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (databaseState.status === 'available') {
            await this.rdsClient
                .stopDBInstance({
                DBInstanceIdentifier: databaseState.identifier,
            })
                .promise();
            task.output = 'Stopped successfully';
        }
        else {
            task.skip(`Skipped, current state is not "available" it is "${databaseState.status}" instead`);
        }
    }
    async restoreDatabase(task, dryRun, databaseState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (databaseState.status === 'available') {
            try {
                task.output = 'Starting RDS instance...';
                await this.rdsClient
                    .startDBInstance({
                    DBInstanceIdentifier: databaseState.identifier,
                })
                    .promise();
                task.output = 'Waiting for RDS instance to be available...';
                await this.rdsClient.waitFor('dBInstanceAvailable', {
                    DBInstanceIdentifier: databaseState.identifier,
                });
                task.output = 'Started successfully';
            }
            catch (error) {
                if (error.code === 'InvalidDBInstanceState') {
                    task.skip('Skipped, database is not in "stopped" state.');
                }
                throw error;
            }
        }
        else {
            task.skip(`Skipped, previous state was not "available" it was "${databaseState.status}" instead`);
        }
    }
    async getCurrentState(databases) {
        return Promise.all(databases.map(async (database) => {
            if (database.DBInstanceIdentifier === undefined) {
                throw new Error(`Unexpected error: DBInstanceIdentifier is missing for RDS database`);
            }
            if (database.DBInstanceStatus === undefined) {
                throw new Error(`Unexpected error: DBInstanceStatus is missing for RDS database`);
            }
            return {
                identifier: database.DBInstanceIdentifier,
                status: database.DBInstanceStatus,
            };
        }));
    }
    async listDatabases() {
        return ((await this.rdsClient.describeDBInstances().promise()).DBInstances || []);
    }
}
exports.StopRdsDatabaseInstancesTrick = StopRdsDatabaseInstancesTrick;
StopRdsDatabaseInstancesTrick.machineName = 'stop-rds-database-instances';
//# sourceMappingURL=stop-rds-database-instances.trick.js.map